;===============================================================================
;file.open
;desc: opens file
;args: name - pointer to null terminated string containing file name
;      mode - 0: read, 1: write, 2:r/w
;ret: CF on error, otherwise
;     EAX = handle to file
;note: don't forget to file.close returned handle!
;error: ERR_FILE_NOT_FOUND - file not found
;       ERR_CANT_OPEN_FILE - file exists, but we failed to open it
;       ERR_INVALID_MODE - invalid mode
;       ERR_UNKNOWN
;===============================================================================
proc fileio.open name, mode
	push	ebx ecx edx

	cmp	[mode], 2
	ja	.error_invalid_mode

	mov	edx, 0		    ;file permissions
	mov	ecx, [mode]	    ;file access bits (same as our modes)
	mov	ebx, [name]	    ;file name pointer
	mov	eax, 5		    ;sys_open
	int	80h

	cmp	eax, -4096
	ja	.error_open

	clc

.r:	pop	edx ecx ebx
	ret
.rc:	stc
	jmp	.r

.error_open:

	;check file not found
	push	ERR_FILE_NOT_FOUND
	cmp	eax, -2 ;ENOENT, ENOFILE
	je	@f
	mov	[esp], eax
@@:	pop	eax
	je	.rc

	;check can't open
	push	ERR_CANT_OPEN_FILE
	cmp	eax, -13 ;EACCES
	je	@f
	cmp	eax, -14 ;EFAULT
	je	@f
	cmp	eax, -21 ;EISDIR
	je	@f
	cmp	eax, -26 ;ETXTBUSY
	je	@f
	cmp	eax, -28 ;ENOSPC
	je	@f
	cmp	eax, -30 ;EROFS
	je	@f
	mov	[esp],eax
@@:	pop	eax
	je	.rc

	mov	eax, ERR_UNKNOWN
	jmp	.rc

.error_invalid_mode:
	mov	eax, ERR_INVALID_MODE
	jmp	.rc
endp