;===============================================================================
;file.read
;desc: read data from current position in file
;args: handle - file handle (returned by file.open)
;      buffer - pointer to buffer where to store data
;      count - number of bytes to read
;ret: CF set on error, otherwise
;     OF set on read behind EOF (less that <count> bytes were readen)
;     EAX = number of bytes readen
;     ZF set if no bytes were readen (we already were at/behind EOF), then EAX=0
;error: ERR_INVALID_HANDLE - invalid handle, or handle is write-only
;       ERR_ZERO_SIZE      - count is zero
;       ERR_UNKNOWN
;===============================================================================
proc fileio.read handle, buffer, count
	push	ebx ecx edx
	pushf

	;error if count=0
	cmp	[count],0
	ja	@f
	mov	eax, ERR_ZERO_SIZE
	jmp	.rc
@@:

	;do the read
	mov	edx, [count]
	mov	ecx, [buffer]
	mov	ebx, [handle]
	mov	eax, 3		;sys_read
	int	80h
	cmp	eax, -4096
	ja	.error_read

	;no error, clear CF
	and	byte [esp],not 1		;clear CF

	;set OF if bytes readen != bytes to read
	and	dword [esp],not (1 shl 11)	;clear OF
	cmp	eax,[count]
	je	@f
	or	dword [esp],1 shl 11		;set OF
@@:

	;set ZF if no bytes were readen
	and	byte [esp],not (1 shl 6)	;clear ZF
	cmp	eax, 0
	ja	@f
	or	byte [esp], 1 shl 6		;set ZF
@@:

	;return number of bytes readen in EAX

.r:	popf
	pop	edx ecx ebx
	ret

	;error occured
.rc:	or	dword [esp], 1 ;set CF
	jmp .r


.error_read:
	;catch EBADF
	cmp	eax, -9; EBADF
	jne	@f
	mov	eax, ERR_INVALID_HANDLE
	stc
	jmp	.rc
@@:

	mov	eax, ERR_UNKNOWN
	jmp	.rc
endp
