;===============================================================================
;file.write
;desc: writes data to current position in file
;args: handle - file handle (returned by file.open)
;      buffer - pointer to buffer from where to read data
;      size - number of bytes to read
;ret: CF set on error
;error: ERR_INVALID_HANDLE - invalid handle
;       ERR_ZERO_SIZE      - count is zero
;       ERR_UNKNOWN
;===============================================================================
proc fileio.write handle, buffer, count
	push	ebx ecx edx

	;error if count=0
	cmp	[count],0
	ja	@f
	mov	eax, ERR_ZERO_SIZE
	jmp	.rc
@@:

	;do the write
	mov	edx, [count]
	mov	ecx, [buffer]
	mov	ebx, [handle]
	mov	eax, 4		;sys_write
	int	80h
	cmp	eax, -4096
	ja	.error_write

	;error if everything wasn't written
	cmp	eax, [count]
	je	@f
	mov	eax, ERR_UNKNOWN
	jmp	.rc
@@:

	;no error, clear CF
	clc
.r:	pop	edx ecx ebx
	ret
.rc:	stc
	jmp	.r

.error_write:
	;catch EBADF
	cmp	eax, -9; EBADF
	jne	@f
	mov	eax, ERR_INVALID_HANDLE
	jmp	.rc
@@:

	mov	eax, ERR_UNKNOWN
	jmp	.r

endp