;this is internal procedure to be used with heap manager's mem.init.
;Don't call this procedure directly.

;; @name mem.alloc_heap
;; @desc
;;    This is internal procedure of heap manager, that handles
;;    platform-dependent aspect of allocating entire heap.
;;    Do not call this procedure.
;; @arg size
;;    Initial size of heap to allocate.
;; @ret
;;    CF set on error, otherwise
;;    EAX = pointer to allocated memory block
;; @err ERR_OUT_OF_MEM
;;     Not enough memory
;; @err ERR_ZERO_SIZE
;;     size = 0
;; @err ERR_OUT_OF_RANGE
;;     size is negative (or larger than 7FFFFFFFh if viewed as unsigned)
;; @err ERR_UNKNOWN
;;     System returned error that wasn't translated to FASMLIB error.
;;     You can {contact} author about this, and translation can be added.
;; @warn Don't call this procedure!
;; @note Returned pointer is always >= 10000h to make it easily distinguishable from handles.
;; @note This procedure is used only on platforms where FASM heap manager is used.
proc mem.alloc_heap  size
	push	ebx ecx

	;error if size=0
	cmp	[size], 0
	je	.error_size_zero

	;error is size>=80000000h
	cmp	[size], 80000000h
	je	.error_size_too_big

	;call brk(0) to find beginning of heap
	mov	ebx, 0
	mov	eax, 45 	;sys_brk
	int	80h
	cmp	eax, -4096
	ja	.error_brk
	mov	ecx, eax

	;call brk(ecx+size) to set up heap size
	mov	ebx, ecx
	add	ebx, [size]	;
	mov	eax, 45 	;sys_brk
	int	80h
	cmp	eax, -4096
	ja	.error_brk

	;return address of heap
	mov	eax, ecx

.rnc:	clc
.r:	pop	ecx ebx
	ret
.rc:	stc
	jmp	.r

.error_brk:

	;catch out-of-memory
	cmp	eax, -12 ;ENOMEM
	jne	@f
	mov	eax, ERR_OUT_OF_MEM
	jmp	.rc
@@:

	mov	eax, ERR_UNKNOWN
	jmp	.rc

.error_size_zero:
	mov	eax, ERR_ZERO_SIZE
	jmp	.rc

.error_size_too_big:
	mov	eax, ERR_OUT_OF_RANGE
	jmp	.rc

endp
