;; @name fileio.create
;; @desc
;;    Creates new file, and returns handle to it.
;; @arg name
;;    Name of file to create.
;; @arg overwr
;;    Sets behavior if file already exists:
;;    1 = overwrites file, 0 = returns ERR_FILE_EXISTS.
;; @arg mode
;;    Sets access mode to file:
;;    1 = write only, 2 = read/write
;; @ret
;;    CF = 1 on error, otherwise
;;    EAX = file handle
;; @err ERR_FILE_EXISTS
;;    File with given name already exists (only returned if mode=0)
;; @err ERR_CANT_OPEN_FILE
;;    File can't be created.
;; @err ERR_INVALID_MODE
;;    mode is not 0 or 1.
;; @err ERR_UNKNOWN
;;    Platform-specific error during creating file.
;; @warn Don't forget to close returned handle with {fileio.close}!
;; @note This is platform-dependent procedure
proc fileio.create name, overwr, mode
	push ecx edx

	;check if mode is 1 or 2
	mov	eax, [mode]
	test	eax, eax
	jz	.error_invalid_mode
	cmp	eax, 2
	ja	.error_invalid_mode

	;check if overwr is <=1
	cmp	[overwr], 1
	ja	.error_invalid_mode

	;push some args
	push	0 ;hTemplateFile
	push	0 ;dwFlagsAndAttributes

	;overwrite existing file?
	cmp	[overwr],0
	je	@f
	push	2	   ;CREATE_ALWAYS
	jmp	.create_mode_done
@@:	push	1	   ;CREATE_NEW
.create_mode_done:

	;push some more junk
	push	0 ;pointer to SECURITY_ATTRIBUTES
	push	0 ;dwShareMode

	;push proper access flag depending on [mode]
	cmp	[mode],1
	je	@f
	push	0C0000000h ;GENERIC_READ | GENERIC_WRITE
	jmp	.access_mode_done
@@:	push	40000000h ;GENERIC_WRITE
.access_mode_done:

	;push file name
	push	[name]

	;call CreateFileA
	call	[CreateFileA]
	cmp	eax, -1
	je	.error_CreateFile

	;no error occured, return system handle
	clc

	;return
.r:	pop	edx ecx
	ret
.rc:	stc
	jmp	.r

	;CreateFile returned with error
.error_CreateFile:
	call	[GetLastError]

	;catch already-exists
	push	ERR_FILE_EXISTS
	cmp	eax, 80 		;ERROR_FILE_EXISTS
	je	@f
	cmp	eax, 183		;ERROR_ALREADY_EXISTS
	je	@f
	mov	[esp],eax
@@:	pop	eax
	je	.rc

	;rest can be classified as can't create
	mov	eax, ERR_CANT_OPEN_FILE
	jmp	.rc

.error_invalid_mode:
	mov	eax, ERR_INVALID_MODE
	jmp	.rc
endp