;; @name fileio.open
;; @desc
;;    Opens existing file.
;; @arg name
;;    Pointer to name of file to open. Must be zero-terminated string.
;; @arg mode
;;    Access mode to file: 0=read only, 1=write only, 2=read/write.
;; @ret
;;    CF = 1 on error, otherwise
;;    EAX = file handle
;; @err ERR_FILE_NOT_FOUND
;;    File with specified name wasn't found.
;; @err ERR_CANT_OPEN_FILE
;;    File exists, but couldn't be opened.
;; @err ERR_INVALID_MODE
;;    mode > 2.
;; @err ERR_UNKNOWN
;;    Platform-specific error during opening file.
;; @warn Don't forget to close returned handle with {fileio.close}!
;; @note This is platform-dependent procedure
proc fileio.open name, mode

	push ecx edx

	;push some args
	push	0		;hTemplateFile
	push	0		;dwFlagsAndAttributes
	push	3		;dwCreationDistributiuon - OPEN_EXISTING
	push	0		;pointer to SECURITY_ATTRIBUTES

	;push share mode depending on [.mode]
	cmp	[mode], 0      ;read
	jne	@f
	push	1		;FILE_SHARE_READ
	jmp	.sharemode_done
@@:	push	0
.sharemode_done:


	;push proper mode flag depending on [.mode]
	cmp	[mode], 0
	jnz	@f
	push	80000000h	;GENERIC_READ
	jmp	.modedone
@@:
	cmp	[mode], 1
	jnz	@f
	push	40000000h	;GENERIC_WRITE
	jmp	.modedone
@@:
	cmp	[mode], 2
	jnz	@f
	push	0C0000000h	;GENERIC_READ | GENERIC_WRITE
	jmp	.modedone
@@:
	mov	eax, ERR_INVALID_MODE
	jmp	.rc
.modedone:

	;push file name
	push	[name]

	;call it
	call	[CreateFileA]
	cmp	eax,-1
	je	.error_CreateFile

	;return withot error
.rnc:	clc

	;return
.r:	pop	edx ecx
	ret

.rc:	stc
	jmp	.r

	;CreateFile returned with error
.error_CreateFile:
	call	[GetLastError]

	;catch file not found errors
	push	ERR_FILE_NOT_FOUND
	cmp	eax, 2			;ERROR_FILE_NOT_FOUND
	je	@f
	cmp	eax, 3			;ERROR_PATH_NOT_FOUND
	je	@f
	cmp	eax, 15 		;ERROR_INVALID_DRIVE
	je	@f
	cmp	eax, 123		;ERROR_INVALID_NAME
	je	@f
	mov	[esp],eax
@@:	pop	eax
	je	.rc

	;catch can't open file errors
	push	ERR_CANT_OPEN_FILE
	cmp	eax, 4			;ERROR_TOO_MANY_OPEN_FILES
	je	@f
	cmp	eax, 5			;ERROR_ACCESS_DENIED  (also returned when trying to "open" directory
	je	@f
	cmp	eax, 19 		;ERROR_WRITE_PROTECT
	je	@f
	cmp	eax, 30 		;ERROR_READ_FAULT
	je	@f
	cmp	eax, 32 		;ERROR_SHARING_VIOLATION
	je	@f
	cmp	eax, 33 		;ERROR_LOCK_VIOLATION
	je	@f
	cmp	eax, 36 		;ERROR_SHARING_BUFFER_EXCEEDED
	je	@f
	cmp	eax, 110		;ERROR_OPEN_FAILED
	je	@f
	mov	[esp],eax
@@:	pop	eax
	je	.rc

	mov	eax, ERR_UNKNOWN
	jmp	.rc
endp