;; @name fileio.read
;; @desc
;;    Reads data from file.
;; @arg handle
;;    File handle (returned by {fileio.open}, {fileio.create} or {fileio.append}).
;; @arg buffer
;;    Pointer to buffer where to read data.
;; @arg count
;;    Number of bytes to read.
;; @ret
;;    CF set on error, otherwise
;;    EAX = number of bytes readen, and
;;    OF set if EOF was reached (less than count bytes read)
;; @err ERR_INVALID_HANDLE
;;    Invalid handle.
;; @err ERR_ZERO_SIZE
;;    count=0.
;; @err ERR_UNKNOWN
;;    Platform-specific error.
;; @note This is platform-dependent procedure
proc fileio.read handle, buffer, count
  locals
    .bytesread dd ?
  endl
	push	ecx edx
	pushf

	;check if count>0
	cmp	[count], 0
	ja	@f
	mov	eax, ERR_ZERO_SIZE
	jmp	.rc
@@:
	;call it
	push	0 ;lpOverlapped
	lea	eax,[.bytesread]
	push	eax
	push	[count]
	push	[buffer]
	push	[handle]
	call	[ReadFile]

	;check for error
	or	eax,eax
	je	.error_ReadFile

	;no error, clear CF
	btr	dword [esp], 0 ;clear CF

	;set OF if bytes readen != bytes to read
	btr	dword [esp], 11 ;clear OF
	mov	eax,[.bytesread]
	cmp	eax,[count]
	je	@f
	bts	dword [esp], 11 ;set OF
@@:

	;return number of bytes readen in EAX
	mov	eax,[.bytesread]

.r:	popf
	pop	edx ecx
	ret

	;error occured
.rc:	bts	dword [esp], 0 ;set CF
	jmp	.r

.error_ReadFile:
	call	[GetLastError]

	push	ERR_INVALID_HANDLE
	cmp	eax, 5 ;ERROR_ACCESS_DENIED    - calling file.read with write-only handle can be considered invalid argument
	je	@f
	cmp	eax, 6 ;ERROR_INVALID_HANDLE
	je	@f
	mov	[esp], eax
@@:	pop	eax
	je	.rc

	mov	eax, ERR_UNKNOWN
	jmp	.rc
endp