;; @name fileio.write
;; @desc
;;    Writes data to file.
;; @arg handle
;;    File handle (returned by {fileio.open}, {fileio.create} or {fileio.append}).
;; @arg buffer
;;    Pointer to data to write.
;; @arg count
;;    Size of daata to write.
;;    Procedure does nothing if this is zero.
;; @ret
;;    CF set on error
;; @err ERR_INVALID_HANDLE
;;    Invalid handle.
;; @err ERR_UNKNOWN
;;    Platform-specific error.
;; @note This is platform-dependent procedure
proc fileio.write handle, buffer, count
  locals
    .byteswritten dd ?
  endl

	push	ecx edx

	;return if count=0
	cmp	[count], 0
	je	.rnc

	;call it
	push	0 ;lpOverlapped
	lea	eax,[.byteswritten]
	push	eax
	push	[count]
	push	[buffer]
	push	[handle]
	call	[WriteFile]

	;check for error
	or	eax,eax
	je	.error_WriteFile

	;check bytes written
	mov	eax,[.byteswritten]
	cmp	eax,[count]
	je	@f
	mov	eax, ERR_UNKNOWN
	jmp	.rc
@@:

	;no error
.rnc:	clc

.r:	pop	edx ecx
	ret
.rc:	stc
	jmp	.r

.error_WriteFile:
	call	[GetLastError]

	;catch invalid handle
	push	ERR_INVALID_HANDLE
	cmp	eax, 6 ;ERROR_INVALID_HANDLE
	je	@f
	mov	[esp],eax
@@:	pop	eax
	je	.rc

	mov	eax, ERR_UNKNOWN
	jmp	.rc
endp