;; @name mem.size
;; @desc
;;    Returns size of memory block allocated by {mem.alloc}, {mem.alloc0},
;;    {mem.realloc}, or {mem.realloc0}.
;; @arg blockptr
;;    Pointer to allocated memory block.
;;    This must be same pointer as returned from allocation function, pointer into block is not enough.
;; @ret
;;    CF set on error, otherwise
;;    EAX = size of allocated block
;; @err ERR_INVALID_POINTER
;;    Blockptr is not a valid allocated block pointer.
;;    This could also mean {heap:heap corruption}, these two cases cannot be properly distinguished.
;; @err ERR_HEAP_CORRUPT
;;     {heap:Heap is corrupted}. You was writing to memory outside of allocated block.
;; @err ERR_MODULE_NOT_INITIALIZED
;;      Module is not initialized. Initialize module with {mem.init} to use this procedure.
;; @err ERR_UNKNOWN
;;     System returned error that wasn't translated to FASMLIB error.
;;     You can {contact} author about this, and translation can be added.
;; @note Memory management must be {mem.init:initialized} for this procedure to work.
;; @note This is platform-specific procedure
proc mem.size blockptr
	push	ecx edx

	;is module initialized?
	cmp	[mem.initialized],0
	je	.error_not_initialized

	;EAX = memory block
	mov	eax, [blockptr]

	;check if it is our memory block
	cmp	dword [eax-8], 'RnR'
	je	@f
	mov	eax, ERR_INVALID_POINTER
	jmp	.rc
@@:

	;return size of block
	mov	eax, [eax-4]
	clc

	clc
.r:	pop	edx ecx
	ret
.rc:	stc
	jmp	.r

.error_not_initialized:
	mov	eax, ERR_MODULE_NOT_INITIALIZED
	jmp	.rc
endp