;; @name extconv.iw_dec
;; @desc
;;    Converts signed word (16bit) value to decimal string.
;;    Supports padding of output with spaces or zeroes, and explicit plus sign.
;;    For simpler version see {conv.iw_dec}.
;; @arg dest
;;    Pointer to destination string buffer.
;; @arg destlen
;;    Size of destination string buffer.
;; @arg value
;;    Word value to convert.
;;    Only low order word of this dword is used, rest is ignored.
;; @arg padding
;;    Type of padding:
;;    Type 0 is no padding.
;;    Type 1 is prepending with spaces, and sign counts into padding size.
;;    Type 2 is prepending with leading zeroes. In this type padlen specifies number
;;    digits displayed. One more character is always printed as sign (' ', '+' or '-').
;; @arg padlen
;;    Length to which should output be padded.
;;    If output is already longer than this value before padding, then output is not padded.
;;    Ignored if padding = 0.
;; @arg plus
;;    Set to 1 if you want to display plus sign on positive numbers.
;; @arg len
;;    Pointer to dword variable, which will on return hold length of output (terminating zero not included).
;;    Ignored if zero.
;; @ret
;;    CF=1 on error.
;; @err ERR_BUFFER_FULL
;;    Destination buffer is not big enough.
;;    Only first destlen bytes of resulting string were written.
;;    String in destination buffer is NOT zero terminated.
;;    Variable pointed by len contains destlen
;; @err ERR_ZERO_SIZE
;;    destlen=0.
;;    Variable pointed by len is set to 0.
;; @err ERR_INVALID_MODE
;;    padding > 2, or plus > 1
;;    Variable pointed by len is set to 0.
;; @nodeps
proc extconv.iw_dec dest, destlen, value, padding, padlen, plus, len
	movsx	eax, word [value]
	libcall extconv.id_dec, [dest], [destlen], eax, [padding], [padlen], [plus], [len]
	ret
endp