;; @name conv.hex_iw
;; @desc
;;    Converts hexadecimal string to signed word (16bit) value.
;; @arg string
;;    Pointer to string to convert.
;; @arg buflen
;;    Size of string's buffer.
;;    You can set this to -1 (FFFFFFFFh) if string is quaranteed to be properly zero-terminated.
;; @ret
;;    CF=1 on error, otherwise
;;    EAX = value.
;; @err ERR_NOT_HEXADECIMAL
;;    First character of string is not hexadecimal digit.
;; @err ERR_OUT_OF_RANGE
;;    Value is lesser than -8000 or greater than 7FFF.
;; @err ERR_ZERO_SIZE
;;    strlen=0
;; @note This procedure converts digits until it reaches non-digit character,
;;       or until value overflows.
;; @note To get number of converted characters, use {extconv.hex_iw}.
;; @nodeps
proc conv.hex_iw string, buflen
	libcall extconv.hex2id_size, [string], [buflen], 8000h, 1, 0
	ret
endp
