;; @name fileio.size
;; @desc
;;    Returns size of opened file. Returned value is 64bit value.
;; @arg handle
;;    File handle (returned by {fileio.open}, {fileio.create} or {fileio.append}).
;; @ret
;;    CF set on error, otherwise
;;    EDX:EAX = size of file in bytes
;; @err ERR_INVALID_HANDLE
;;    Invalid file handle.
;; @err ERR_UNKNOWN
;; @note This is platform-dependent procedure
proc fileio.size handle
local pos:QWORD
local size:QWORD

	push	ebx

	;EBX = file handle
	mov	ebx, [handle]

	;save current position in file
	libcall fileio.tell, ebx
	jc	.rc
	mov	dword [pos], eax
	mov	dword [pos+4], edx

	;seek end of file
	libcall fileio.seek, ebx, 0, 0, 2
	jc	.rc

	;get position of end of file
	libcall fileio.tell, ebx
	jc	.rc
	mov	dword [size], eax
	mov	dword [size+4], edx

	;restore original position
	libcall fileio.seek, ebx, dword [pos], dword [pos+4], 0
	jc	.rc

	;return position of end of file
	mov	eax, dword [size]
	mov	edx, dword [size+4]

.rnc:	clc
.r:	pop	ebx
	ret
.rc:	stc
	jmp	.r
endp