;; @name fstream._close
;; @desc
;;    This is {stream close method:close method} of {fstream:file stream}.
;;    Don't call this procedure directly, use {file.close} instead.
;; @arg stream
;;    Handle of file stream to write to.
;; @ret
;;    CF set on error
;; @err ERR_HANDLE_CORRUPT
;;    Stream is corrupted.
;;    This can happen if you close file handle used by stream.
;;    This can also happen if you overwrite stream object.
;; @err ERR_UNKNOWN
;;    Platform-specific error.
proc fstream._close stream
	mov	eax, [stream]

	;should we close handle?
	cmp	[eax + FSTREAM.closeh], 1
	jne	.rnc

	;yes, let's do so
	mov	eax, [eax + FSTREAM.handle]
	libcall fileio.close, eax
	jc	.error_close

.rnc:	clc
.r:	ret
.rc:	stc
	jmp	.r

.error_close:

	;translate ERR_INVALID_HANDLE to ERR_HANDLE_CORRUPT.
	;if file handle in stream is invalid, them stream object is corrupted
	cmp	eax, ERR_INVALID_HANDLE
	jne	@f
	mov	eax, ERR_HANDLE_CORRUPT
	jmp	.rc
	@@:

	jmp	.rc

endp
