;; @name fstream._read
;; @desc
;;    This is {stream read method:read method} of {fstream:file stream}.
;;    Don't call this procedure directly, use {file.read} instead.
;; @arg stream
;;    Handle of file stream to read from.
;; @arg buffer
;;    Pointer to buffer where we want to read data.
;; @arg count
;;    Number of bytes to read.
;; @ret
;;    CF set on error, otherwise
;;    OF set if EOF was reached (less than count bytes readen), and
;;    EAX = number of bytes readon.
;; @err ERR_HANDLE_CORRUPT
;;    Stream is corrupted.
;;    This can happen if you close file handle used by stream.
;;    This can also happen if you overwrite stream object.
;; @err ERR_HANDLE_NOT_READABLE
;;    Stream is not readable.
;; @err ERR_ZERO_SIZE
;;    count = 0.
;; @err ERR_UNKNOWN
;;    Platform-specific error during writing to file.
;; @nodep stream._def_read
proc fstream._read stream, buffer, count

	;check if mode is 0 or 2
	mov	eax, [stream]
	mov	eax, [eax + FSTREAM.mode]
	cmp	eax, 1
	je	.error_stream_not_readable

	;call default read method
	libcall stream._def_read, [stream], [buffer], [count], fstream._readbuf
	jc	.error_read

.r:	ret
.rc:	stc
	jmp	.r

.error_stream_not_readable:
	mov	eax, ERR_HANDLE_NOT_READABLE
	jmp	.rc

.error_read:

	;translate ERR_INVALID_HANDLE to ERR_HANDLE_CORRUPT.
	;if file handle in stream is invalid, them stream object is corrupted
	cmp	eax, ERR_INVALID_HANDLE
	jne	@f
	mov	eax, ERR_HANDLE_CORRUPT
	jmp	.rc
	@@:

	jmp	.rc

endp