;; @name fstream._seek
;; @desc
;;    This is {stream seek method:seek method} of {fstream:file stream}.
;;    Don't call this procedure directly, use {file.seek} instead.
;; @arg stream
;;    Handle of file stream.
;; @arg distlow
;;    Low order dword of distance to move.
;; @arg disthigh
;;    High order dword of distance to move.
;; @arg mode
;;    Mode of seeking.
;;    0 = seek from beginning.
;;    1 = seek relative distance from current position.
;;    2 = seek from end (distance should be negative value).
;; @ret
;;    CF set on error
;; @err ERR_HANDLE_CORRUPT
;;    Stream is corrupted.
;;    This can happen if you close file handle used by stream.
;;    This can also happen if you overwrite stream object.
;; @err ERR_UNKNOWN
;;    Platform-specific error.
proc fstream._seek stream, distlow, disthigh, mode
	push	ebx edx

	;EBX = stream object
	mov	ebx, [stream]

	;if we are seeking relative distance from current position (mode 1),
	;we must subtract size of data in read buffer from seek distance
	cmp	[mode], 1
	jne	@f
	mov	eax, [ebx + FSTREAM.rbuf.cnt]
	sub	[distlow], eax
	sbb	[disthigh], 0
	@@:

	;seek desired position
	libcall fileio.seek, [ebx + FSTREAM.handle], [distlow], [disthigh], [mode]
	jc	.error_seek

	;invalidate data in read buffer
	mov	[ebx + FSTREAM.rbuf.cnt], 0
	mov	[ebx + FSTREAM.rbuf.pos], 0

.r:	pop	edx ebx
	ret
.rc:	stc
	jmp	.r

.error_seek:

	;translate ERR_INVALID_HANDLE to ERR_HANDLE_CORRUPT.
	;if file handle in stream is invalid, them stream object is corrupted
	cmp	eax, ERR_INVALID_HANDLE
	jne	@f
	mov	eax, ERR_HANDLE_CORRUPT
	jmp	.rc
	@@:

	jmp	.rc
endp