;; @name fstream._tell
;; @desc
;;    This is {stream tell method:tell method} of {fstream:file stream}.
;;    Don't call this procedure directly, use {file.tell} instead.
;; @arg stream
;;    Handle of stream.
;; @ret
;;    CF set on error, otherwise
;;    EDX:EAX = new position in file
;; @err ERR_HANDLE_CORRUPT
;;    Stream is corrupted.
;;    This can happen if you close file handle used by stream.
;;    This can also happen if you overwrite stream object.
;; @err ERR_UNKNOWN
;;    Platform-specific error.
proc fstream._tell stream
	push	ebx

	mov	ebx, [stream]

	;get real file position
	libcall fileio.tell, [ebx + FSTREAM.handle]
	jc	.error_seek

	;subtract size of data in read buffer
	sub	eax, [ebx + FSTREAM.rbuf.cnt]
	sbb	edx, 0

.rnc:	clc
.r:	pop	ebx
	ret
.rc:	stc
	jmp	.r

.error_seek:

	;translate ERR_INVALID_HANDLE to ERR_HANDLE_CORRUPT.
	;if file handle in stream is invalid, them stream object is corrupted
	cmp	eax, ERR_INVALID_HANDLE
	jne	@f
	mov	eax, ERR_HANDLE_CORRUPT
	jmp	.rc
	@@:

	jmp	.rc
endp