;; @name file.append
;; @desc
;;    Opens existing file for writing, and sets pointer to end of file.
;; @arg name
;;    Name of file to create
;; @arg create
;;    Specifies whether to create file if it doesn't exist.
;;    1=create, 0=don't create.
;; @arg mode
;;    Access mode to file: 1=write only, 2=read/write.
;; @ret
;;    CF = 1 on error, otherwise
;;    EAX = stream handle
;; @err ERR_FILE_NOT_FOUND
;;    File with given name wasn't found.
;;    Only returned when create=0.
;; @err ERR_CANT_OPEN_FILE
;;    File can't be created.
;; @err ERR_INVALID_MODE
;;    mode = 0, or mode > 2, or overwr > 1.
;; @err ERR_UNKNOWN
;;    Platform-specific error during creating file.
;; @warn Don't forget to close returned file stream with {stream.close}!
;; @note This is simpler version, see {file.append} for more options.
;; @note Opens file as stream, if you want direct file I/O use {fileio.append}.
proc file.append name, create, mode
	;check if mode is valid
	cmp	[mode], 2
	ja	.error_invalid_mode

	;we need read buffer if mode is 0 or 2
.check_read_buffer:
	cmp	[mode], 1
	je	.no_read_buffer
	push	dword FSTREAM_READ_BUFFER_SIZE
	push	dword FSTREAM_READ_BUFFER
	jmp	.read_buffer_done
.no_read_buffer:
	push	dword 0
	push	dword 0
.read_buffer_done:

	;we need write buffer if mode is 1 or 2
.check_write_buffer:
	cmp	[mode], 0
	je	.no_write_buffer
	push	dword FSTREAM_WRITE_BUFFER_SIZE
	push	dword FSTREAM_WRITE_BUFFER
	jmp	.write_buffer_done
.no_write_buffer:
	push	dword 0
	push	dword 0
.write_buffer_done:

	;call extended version
	libcall file.extappend, 0, [name], [create], [mode]
	jc	.rc

.rnc:	clc
.r:	ret
.rc:	stc
	jmp	.r

.error_invalid_mode:
	mov	eax, ERR_INVALID_MODE
	jmp	.rc
endp
