;; @name file.create
;; @desc
;;    Creates new file, and returns stream handle.
;; @arg name
;;    Name of file to create
;; @arg overwr
;;    Sets behavior if file already exists:
;;    1 = overwrites file, 0 = returns ERR_FILE_EXISTS
;; @arg mode
;;    Sets access mode:
;;    1 = write only, 2 = read/write
;; @ret
;;    CF = 1 on error, otherwise
;;    EAX = stream handle
;; @err ERR_FILE_EXISTS
;;    File with given name already exists (only returned if overwr=0)
;; @err ERR_CANT_OPEN_FILE
;;    File can't be created.
;; @err ERR_INVALID_MODE
;;    mode = 0, or mode > 2, or overwr > 1
;; @err ERR_UNKNOWN
;;    Platform-specific error during creating file.
;; @warn Don't forget to close returned file stream with {stream.close}!
;; @note This is simple version, see {file.extcreate} for more options.
;; @note Opens file as stream, if you want direct file I/O use {fileio.create}.
proc file.create name, overwr, mode

	;check if mode is valid
	cmp	[mode], 2
	ja	.error_invalid_mode

	;we need read buffer if mode is 0 or 2
.check_read_buffer:
	cmp	[mode], 1
	je	.no_read_buffer
	push	dword FSTREAM_READ_BUFFER_SIZE
	push	dword FSTREAM_READ_BUFFER
	jmp	.read_buffer_done
.no_read_buffer:
	push	dword 0
	push	dword 0
.read_buffer_done:

	;we need write buffer if mode is 1 or 2
.check_write_buffer:
	cmp	[mode], 0
	je	.no_write_buffer
	push	dword FSTREAM_WRITE_BUFFER_SIZE
	push	dword FSTREAM_WRITE_BUFFER
	jmp	.write_buffer_done
.no_write_buffer:
	push	dword 0
	push	dword 0
.write_buffer_done:

	;call extended version
	libcall file.extcreate, 0, [name], [overwr], [mode]
	jc	.rc

.rnc:	clc
.r:	ret
.rc:	stc
	jmp	.r

.error_invalid_mode:
	mov	eax, ERR_INVALID_MODE
	jmp	.rc
endp