;; @name file.open.handle
;; @desc
;;    Opens file handle as file stream.
;; @arg objbuf
;;    Pointer to buffer for {fstream:FSTREAM} object.
;;    If 0, object is allocated.
;; @arg handle
;;    File handle returned by {fileio.open}, {fileio.create}, or {fileio.append}.
;; @arg mode
;;    Access mode to file: 0=read only, 1=write only, 2=read/write.
;;    This mode should be same as one specified when opening handle.
;;    If mode for handle was 2, this can be 0 or 1, but not otherwise.
;; @ret
;;    CF = 1 on error, otherwise
;;    EAX = stream handle
;; @err ERR_INVALID_HANDLE
;;    Invalid file handle passed
;; @err ERR_INVALID_MODE
;;    mode > 2.
;; @err ERR_UNKNOWN
;;    Platform-specific error during opening file.
;; @warn Don't forget to close returned file stream with {stream.close}!
;; @warn Do not use original handle while it is used by stream.
;; @warn When you close file stream created with this procedure, file handle
;;       still remains open and must be closed with {fileio.close}.
;; @note This is simple version, see {file.extopen.handle} for more options.
proc file.open.handle handle, mode

	;check if mode is valid
	cmp	[mode], 2
	ja	.error_invalid_mode

	;we need read buffer if mode is 0 or 2
.check_read_buffer:
	cmp	[mode], 1
	je	.no_read_buffer
	push	dword FSTREAM_READ_BUFFER_SIZE
	push	dword FSTREAM_READ_BUFFER
	jmp	.read_buffer_done
.no_read_buffer:
	push	dword 0
	push	dword 0
.read_buffer_done:

	;we need write buffer if mode is 1 or 2
.check_write_buffer:
	cmp	[mode], 0
	je	.no_write_buffer
	push	dword FSTREAM_WRITE_BUFFER_SIZE
	push	dword FSTREAM_WRITE_BUFFER
	jmp	.write_buffer_done
.no_write_buffer:
	push	dword 0
	push	dword 0
.write_buffer_done:

	;call extended version
	libcall file.extopen.handle, 0, [handle], [mode]
	jc	.rc

.rnc:	clc
.r:	ret
.rc:	stc
	jmp	.r

.error_invalid_mode:
	mov	eax, ERR_INVALID_MODE
	jmp	.rc
endp