;internal debugging procedure. displays state of heap
proc mem.display_heap stream
local unused:DWORD
local used:DWORD
local allocated:DWORD
	push	ebx edx

	;is module initialized?
	cmp	[mem.initialized], 0
	je	.error_not_initialized

	;was heap corruption detected?
	cmp	[mem.heap_corrupt], 0
	jne	.error_heap_corrupt

	;display heap info
	libcall text.write.format, [stream], <"Heap at %08h, size=%h",10>, [mem.heap.start], [mem.heap.size]
	jc	.rc

	mov	[unused], 0
	mov	[used], 0
	mov	[allocated], 0
	mov	ebx, [mem.heap.start]
	mov	edx, 0

.process_block:

	;check if block lies in heap
	mov	eax, ebx
	sub	eax, [mem.heap.start]
	jb	.error_heap_corrupt
	cmp	eax, [mem.heap.size]
	jae	.error_heap_corrupt

	;check block
	cmp	[ebx + HEAPBLOCK.sign], 'HB'
	jne	.error_heap_corrupt
	cmp	[ebx + HEAPBLOCK.prev], edx
	jne	.error_heap_corrupt

	;if used, check 0 < asize <= size
	cmp	[ebx + HEAPBLOCK.used], 1
	ja	.error_heap_corrupt
	jb	@f
	mov	eax, [ebx + HEAPBLOCK.asize]
	test	eax, eax
	je	.error_heap_corrupt
	cmp	eax, [ebx + HEAPBLOCK.size]
	ja	.error_heap_corrupt
	@@:

	;if last, check if block fills entire heap
	cmp	[ebx + HEAPBLOCK.last], 1
	ja	.error_heap_corrupt
	jb	@f
	lea	eax, [ebx + sizeof.HEAPBLOCK]
	add	eax, [ebx + HEAPBLOCK.size]
	sub	eax, [mem.heap.start]
	cmp	eax, [mem.heap.size]
	jne	.error_heap_corrupt
	@@:

	;update vars
	cmp	[ebx+HEAPBLOCK.used], 1
	je	@f
	mov	eax, [ebx + HEAPBLOCK.size]
	add	[unused], eax
	jmp	.update_done
	@@:
	mov	eax, [ebx + HEAPBLOCK.size]
	add	[used], eax
	mov	eax, [ebx + HEAPBLOCK.asize]
	add	[allocated], eax

.update_done:

	;display info
	mov	eax, ebx
	sub	eax, [mem.heap.start]
	libcall text.write.format, [stream], "Block at %08h (+%h), size = %h", ebx, eax, [ebx+HEAPBLOCK.size]
	jc	.rc

	cmp	[ebx + HEAPBLOCK.used], 1
	jne	@f
	libcall text.write.format, [stream], ", allocsize=%h", [ebx+HEAPBLOCK.asize]
	jc	.rc
	@@:

	libcall text.write, [stream], <10,0>
	jc	.rc

	;if this was last block, end
	cmp	[ebx + HEAPBLOCK.last], 1
	je	.done

	;move to next block
	mov	edx, ebx
	add	ebx, [ebx + HEAPBLOCK.size]
	add	ebx, sizeof.HEAPBLOCK
	jmp	.process_block

.done:
	;edx = padding percentual
	mov	edx, 0
	mov	eax, [used]
	cmp	eax, 0
	je	@f
	sub	eax, [allocated]
	mov	ebx, 100
	mul	ebx
	mov	ebx, [used]
	div	ebx
	mov	edx, eax
	@@:

	mov	eax, [used]
	sub	eax, [allocated]
	libcall text.write.format, [stream],\
		<"unused space = %h, used space = %h, allocated space = %h",10,\
		 "padding bytes = %h (%d\%)",10\
		>,\
		[unused], [used], [allocated],\
		eax, edx
	jc	.rc


.rnc:	clc
.r:	pop	edx ebx
	ret
.rc:	stc
	jmp	.r

.error_not_initialized:
	mov	eax, ERR_MODULE_NOT_INITIALIZED
	jmp	.rc

.error_heap_corrupt:
	mov	[mem.heap_corrupt], 1
	mov	eax, ERR_HEAP_CORRUPT
	jmp	.rc
endp