;=============================================================================
; mem.size
; desc: returns size of memory block
; args: blockptr - pointer to memory block returned by mem.alloc
; ret: CF set on error, orherwise
;      eax = size of memory block
; error: ERR_INVALID_POINTER - blockptr isn't a valid block pointer
;        ERR_HEAP_CORRUPT    - you was overwriting memory outside allocated block
;        ERR_UNKNOWN
;        ERR_MODULE_NOT_INITIALIZED
;=============================================================================
proc mem.size blockptr
	push	ebx

	;is module initialized?
	cmp	[mem.initialized], 0
	je	.error_not_initialized

	;was heap corruption detected?
	cmp	[mem.heap_corrupt], 0
	jne	.error_heap_corrupt

	;EBX = pointer to heap block
	mov	ebx, [blockptr]
	sub	ebx, sizeof.HEAPBLOCK

	;check if pointer lies inside heap
	mov	eax, ebx
	sub	eax, [mem.heap.start]
	jb	.error_bad_pointer
	sub	eax, [mem.heap.size]
	jae	.error_bad_pointer

	;check block
	cmp	[ebx + HEAPBLOCK.sign], 'HB'
	jne	.error_bad_pointer
	cmp	[ebx + HEAPBLOCK.used], 1
	ja	.error_heap_corrupt
	jb	.error_bad_pointer
	cmp	[ebx + HEAPBLOCK.last], 1
	ja	.error_heap_corrupt

	;return size
	mov	eax, [ebx + HEAPBLOCK.asize]
	cmp	eax, [ebx + HEAPBLOCK.size]
	ja	.error_heap_corrupt

.rnc:	clc
.r:	pop	ebx
	ret
.rc:	stc
	ret

.error_not_initialized:
	mov	eax, ERR_MODULE_NOT_INITIALIZED
	jmp	.rc

.error_heap_corrupt:
	mov	[mem.heap_corrupt], 1
	mov	eax, ERR_HEAP_CORRUPT
	jmp	.rc

.error_bad_pointer:
	mov	eax, ERR_INVALID_POINTER
	jmp	.rc
endp