;; @name mem.alloc0
;; @desc
;;    Allocates block of memory, and fills it with zeroes.
;; @arg size
;;    Size of memory to allocate
;; @ret 
;;    CF set on error, otherwise
;;    EAX = pointer to beginning of allocated memory block
;; @err ERR_OUT_OF_MEM
;;    Not enough memory
;; @err ERR_ZERO_SIZE
;;    size = 0
;; @err ERR_OUT_OF_RANGE
;;     size is negative (or larger than 7FFFFFFFh if viewed as unsigned)
;; @err ERR_HEAP_CORRUPT
;;    {heap:Heap is corrupted}. You was writing to memory outside of allocated block.
;; @err ERR_MODULE_NOT_INITIALIZED
;;    Module is not initialized. Initialize module with {mem.init} to use this procedure.
;; @err ERR_UNKNOWN
;;    System returned error that wasn't translated to FASMLIB error.
;;    You can {contact} author about this, and translation can be added.
;; @warn Don't forget to release allocated block with {mem.free}!
;; @note returned pointer must be always >= 10000h
;; @note Returned pointer is always >= 10000h to make it easily distinguishable from handles.
;; @note Maximal allowed size is 7FFFFFFFh, to catch errors with negative values.
;; @note Altough returned block may have bigger size than requested on some implementations,
;;       don't use this additional memory.
;;       It is not zeroes, an {mem.realloc} doesn't preserve this memory.
;; @note Memory management must be {mem.init:initialized} for this procedure to work.
;; @note This is platform-specific procedure
proc mem.alloc0 size

	push	ebx

	;error if size=0
	cmp	[size], 0
	je	.error_size_zero

	;allocate
	libcall mem.alloc, [size]
	jc	.rc
	mov	ebx, eax

	;zero memory
	libcall mem.fill, ebx, [size], 0
	jc	.rc

	mov	eax, ebx

.rnc:	clc
.r:	pop	ebx
	ret
.rc:	stc
	jmp	.r

.error_size_zero:
	mov	eax, ERR_ZERO_SIZE
	jmp	.rc
endp