;============================================================================
;; @name mem.fill
;; @desc
;;    Fills block of memory with given dword pattern.
;; @arg dest
;;    Pointer to block which we will fill.
;; @arg size
;;    Size of block to fill.
;;    If size isn't multiple of 4, remaining bytes will be filled with according
;;    bytes of filler (little endian).
;; @arg value
;;    Dword value with which the block is filled.
;; @ret
;;    CF = 1 on error, otherwise
;;    EAX = dest
;; @err ERR_INVALID_POINTER
;;    dest points to inaccessible area
;; @note Does nothing when size=0
;; @note Memory management doesn't have to be initalized for this to work
proc mem.fill, dest, size, value
	push	ecx edi
	pushf
	cld

	mov	edi, [dest]
	mov	ecx, [size]
	mov	eax, [value]

	;just return if size=0
	cmp	ecx,0
	je	.done

	;align filling pointer to dword
	test	edi, 1b
	jz	@f
	stosb
	ror	eax, 8
	dec	ecx
@@:	test	edi, 10b
	jz	@f
	stosw
	ror	eax, 16
	dec	ecx
	dec	ecx
@@:
	;do the main fill
	push	ecx
	shr	ecx, 2
	rep	stosd
	pop	ecx

	;fill rest
	shr	ecx, 1
	jnc	@f
	stosb
	ror	eax, 8
@@:	shr	ecx, 1
	jnc	@f
	stosw
@@:
	;everything is okay
.done:	and	byte [esp],not 1  ;clear CF in stack
	mov	eax, [dest]

.r:	popf
	pop	edi ecx
	ret
endp