;; @name mstream._peek
;; @desc
;;    This is {stream peek method:peek method} of {mstream:memory stream}.
;;    Don't call this procedure directly, use {stream.peek} instead.
;; @arg stream
;;    Handle of memory stream.
;; @arg count
;;    Number of bytes to peek.
;; @arg readlen
;;    Pointer to variable, which will hold number of chars in returned buffer.
;;    Always returns all bytes up to end of buffer.
;; @ret
;;    CF set on error, otherwise
;;    OF set if EOF was reached (less than count bytes readen), and
;;    EAX = number of bytes readon.
;; @err ERR_ZERO_SIZE
;;    count = 0.
proc mstream._peek stream, count, readlen
	push	ebx ecx esi
	pushf

	;EBX = stream
	mov	ebx, [stream]

	;ECX = count, check if 0
	mov	ecx, [count]
	test	ecx, ecx
	jz	.error_count_zero

	;ESI = position in buffer
	mov	esi, dword [ebx + MSTREAM.pos]

	;return OF=1/EAX=0 if we are outside buffer
	cmp	dword [ebx + MSTREAM.pos+4],0
	jne	.outside_buffer
	cmp	esi, [ebx + MSTREAM.bufsz]
	jae	.outside_buffer

	;ECX = size that we can actually peek = bufsz-pos
	mov	ecx, [ebx + MSTREAM.bufsz]
	sub	ecx, esi

	;return data pointer in EAX
	mov	eax, [ebx + MSTREAM.buf]
	add	eax, esi

	;set readlen to ECX
	cmp	[readlen], 0
	je	@f
	mov	eax, [readlen]
	mov	[eax], ecx
	@@:

	;return OF if there wasn't enough data
	cmp	ecx, [count]
	jne	.ro

.rno:	btr	dword [esp], 11 ;clear OF
.rnc:	btr	dword [esp], 0 ;clear CF
.r:	popf
	pop	esi ecx ebx
	ret
.rc:	bts	dword [esp], 0	;set CF
	jmp	.r
.ro:	bts	dword [esp], 11 ;set OF
	jmp	.rnc

.outside_buffer:
	mov	eax, 0
	jmp	.ro

.error_count_zero:
	mov	eax, ERR_ZERO_SIZE
	jmp	.rc

endp