;; @name mstream._read
;; @desc
;;    This is {stream read method:read method} of {mstream:memory stream}.
;;    Don't call this procedure directly, use {stream.read} instead.
;; @arg stream
;;    Handle of memory stream.
;; @arg buffer
;;    Pointer to buffer where to store readen data.
;; @arg count
;;    Number of bytes to read.
;; @ret
;;    CF set on error, otherwise
;;    OF set if EOF was reached (less than count bytes readen), and
;;    EAX = number of bytes readon.
;; @err ERR_ZERO_SIZE
;;    count = 0.
proc mstream._read stream, buffer, count
	push	ebx ecx esi
	pushf

	;EBX = stream
	mov	ebx, [stream]

	;ECX = count, check if 0
	mov	ecx, [count]
	test	ecx, ecx
	jz	.error_count_zero

	;ESI = position in buffer
	mov	esi, dword [ebx + MSTREAM.pos]

	;return OF=1/EAX=0 if we are outside buffer
	cmp	dword [ebx + MSTREAM.pos+4],0
	jl	.error_internal ;we shouldn't be able to seek to negative file offset
	jg	.outside_buffer
	cmp	esi, [ebx + MSTREAM.bufsz]
	jae	.outside_buffer

	;ECX = size that we can actually read = min(count, bufsz-pos)
	mov	eax, [ebx + MSTREAM.bufsz]
	sub	eax, esi
	cmp	ecx, eax
	cmova	ecx, eax

	;copy data
	add	esi, [ebx + MSTREAM.buf]
	libcall mem.copy, [buffer], esi, ecx
	jc	.rc

	;update pointer
	add	dword [ebx + MSTREAM.pos], ecx

	;return OF if there wasn't enough data
	cmp	ecx, [count]
	jne	.ro

.rno:	btr	dword [esp], 11 ;clear OF
.rnc:	btr	dword [esp], 0 ;clear CF
.r:	popf
	pop	esi ecx ebx
	ret
.rc:	bts	dword [esp], 0	;set CF
	jmp	.r
.ro:	bts	dword [esp], 11 ;set OF
	jmp	.rnc

.outside_buffer:
	mov	eax, 0
	jmp	.ro

.error_internal:
	mov	eax, ERR_FASMLIB
	jmp	.r

.error_count_zero:
	mov	eax, ERR_ZERO_SIZE
	jmp	.rc

endp