;; @name mstream._seek
;; @desc
;;    This is {stream seek method:seek method} of {fstream:file stream}.
;;    Don't call this procedure directly, use {stream.seek} instead.
;; @arg stream
;;    Handle of stream.
;; @arg distlow
;;    Low order dword of distance to move.
;; @arg disthigh
;;    High order dword of distance to move.
;; @arg mode
;;    Mode of seeking.
;;    0 = seek from beginning.
;;    1 = seek relative distance from current position.
;;    2 = seek from end (distance should be negative value).
;; @err ERR_NEGATIVE_SEEK
;;    Seek to negative file offset.
;; @ret
;;    CF set on error
proc mstream._seek stream, distlow, disthigh, mode
	push	ebx edx

	mov	ebx, [stream]

	;check mode
	cmp	[mode], 1
	jb	.mode0
	je	.mode1
	ja	.mode2

	;seek from beginning
.mode0: mov	edx, [disthigh]
	cmp	edx, 0
	jl	.error_negative_seek
	mov	eax, [distlow]
	jmp	.done

	;seek relative
.mode1: mov	eax, dword [ebx + MSTREAM.pos]
	mov	edx, dword [ebx + MSTREAM.pos+4]
	add	eax, [distlow]
	adc	edx, [disthigh]
	cmp	edx, 0
	jl	.error_negative_seek
	jmp	.done

	;seek from end
.mode2: mov	eax, dword [ebx + MSTREAM.bufsz]
	mov	edx, 0
	add	eax, [distlow]
	adc	edx, [disthigh]
	cmp	edx, 0
	jl	.error_negative_seek

.done:	mov	dword [ebx + MSTREAM.pos], eax
	mov	dword [ebx + MSTREAM.pos+4], edx
.rnc:	clc
.r:	pop	edx ebx
	ret
.rc:	stc
	jmp	.r

.error_negative_seek:
	mov	eax, ERR_NEGATIVE_SEEK
	jmp	.rc
endp