;; @name mstream._skip
;; @desc
;;    This is {stream skip method:skip method} of {mstream:memory stream}.
;;    Don't call this procedure directly, use {stream.skip} instead.
;; @arg stream
;;    Handle of memory stream.
;; @arg count
;;    Number of bytes to skip.
;; @ret
;;    CF set on error, otherwise
;;    OF set if EOF was reached (less than count bytes skipped), and
;;    EAX = number of bytes skipped.
proc mstream._skip stream, count
	push	ebx ecx esi
	pushf

	;EBX = stream
	mov	ebx, [stream]

	;ESI = position in buffer
	mov	esi, dword [ebx + MSTREAM.pos]

	;return OF=1/EAX=0 if we are outside buffer
	cmp	dword [ebx + MSTREAM.pos+4],0
	jl	.error_internal ;we shouldn't be able to seek to negative file offset
	jg	.outside_buffer
	cmp	esi, [ebx + MSTREAM.bufsz]
	jae	.outside_buffer

	;ECX = size that we can actually skip = min(count, bufsz-pos)
	mov	eax, [ebx + MSTREAM.bufsz]
	sub	eax, esi
	cmp	ecx, eax
	cmova	ecx, eax

	;skip data
	add	dword [ebx + MSTREAM.pos], ecx

	;return OF=0 if we skipped all remaining data
	cmp	ecx, [count]
	jne	.ro

.rno:	btr	dword [esp], 11 ;clear OF
.rnc:	btr	dword [esp], 0 ;clear CF
.r:	popf
	pop	esi ecx ebx
	ret
.rc:	bts	dword [esp], 0	;set CF
	jmp	.r
.ro:	bts	dword [esp], 11 ;set OF
	jmp	.rnc

.outside_buffer:
	mov	eax, 0
	jmp	.ro

.error_internal:
	mov	eax, ERR_FASMLIB
	jmp	.r

endp