;; @name mstream._write
;; @desc
;;    This is {stream write method:write method} of {mstream:memory stream}.
;;    Don't call this procedure directly, use {stream.write} instead.
;; @arg stream
;;    Handle of memory stream.
;; @arg buffer
;;    Pointer to data which we want to write.
;; @arg count
;;    Number of bytes to write.
;; @ret
;;    CF set on error, otherwise
;;    OF set if EOF was reached (less than count bytes readen), and
;;    EAX = number of bytes readon.
;; @err ERR_BUFFER_FULL
;;    Memory buffer is full. No more place to write.
;; @err ERR_ZERO_SIZE
;;    count = 0.
proc mstream._write stream, buffer, count
	push	ebx ecx esi

	;EBX = stream
	mov	ebx, [stream]

	;ESI = position in buffer
	mov	esi, dword [ebx + MSTREAM.pos]

	;ECX = number of bytes to write
	mov	ecx, [count]

	;return ERR_BUFFER_FULL if we are outside buffer
	cmp	dword [ebx + MSTREAM.pos+4],0
	jne	.error_buffer_full
	cmp	esi, [ebx + MSTREAM.bufsz]
	jae	.error_buffer_full

	;ECX = size that we can actually write = min(count, bufsz-pos)
	mov	eax, [ebx + MSTREAM.bufsz]
	sub	eax, esi
	cmp	ecx, eax
	cmova	ecx, eax

	;copy data
	add	esi, [ebx + MSTREAM.buf]
	libcall mem.copy, esi, [buffer], ecx
	jc	.rc

	;update pointer
	add	dword [ebx + MSTREAM.pos], ecx

	;error if we didn't have space to write everything
	cmp	ecx, [count]
	jb	.error_buffer_full

.rnc:	clc
.r:	pop	esi ecx ebx
	ret
.rc:	stc
	jmp	.r

.error_buffer_full:
	mov	eax, ERR_BUFFER_FULL
	jmp	.rc

endp