;; @name str.poschar
;; @desc
;;    Finds position of first occurence of character inside another string.
;; @arg string
;;    Pointer to string in which we are searching.
;; @arg buflen
;;    Size of buffer of string in which we are searching.
;;    You can set this to -1 (FFFFFFFFh) if string is quaranteed to be properly zero-terminated.
;; @arg char
;;    Character which we are looking for.
;; @ret
;;    CF set on error, otherwise
;;    ZF set if character wasn't found, otherwise
;;    EAX = index of first occurence of character within string.
;; @err ERR_ZERO_VALUE
;;    char=0.
;; @err ERR_ZERO_SIZE
;;    buflen=0.
proc str.poschar string, buflen, char
	push	esi ecx
	pushf	;to return flags

	;ESI = string
	mov	esi, [string]

	;ECX = buflen, error if 0
	mov	ecx, [buflen]
	cmp	ecx, 0
	je	.error_buflen_zero

	;AH = char for which we search, error if 0
	mov	ah, byte [char]
	cmp	ah, 0
	je	.error_char_zero

	;look for character
.search:
	mov	al,[esi]
	inc	esi
	or	al,al
	jz	.not_found

	cmp	al,ah
	je	.found

	dec	ecx
	jnz	.search

.not_found:
	or	dword [esp],(1 shl 6) ;set ZF in stack
	jmp	.rnc

.found:
	and	dword [esp],not (1 shl 6) ;clear ZF
	mov	eax, [buflen]
	sub	eax, ecx

.rnc:	and	dword [esp],not 1 ;clear CF
.r:	popf
	pop	ecx esi
	ret

.rc:	or	dword [esp],1 ;set CF
	jmp	.r

.error_buflen_zero:
	mov	eax, ERR_ZERO_SIZE
	jmp	.rc

.error_char_zero:
	mov	eax, ERR_ZERO_VALUE
	jmp	.rc

endp