;; @name str.upcase
;; @desc
;;    Converts characters in string to upcase.
;;    Works only for ASCII characters (a-z and A-Z), no other encodings are supported.
;; @arg string
;;    Pointer to string.
;; @arg buflen
;;    Size of string buffer.
;;    You can set this to -1 (FFFFFFFFh) if string is quaranteed to be properly zero-terminated.
;; @ret
;;    CF set on error, otherwise
;;    EAX = string.
;; @err ERR_ZERO_SIZE
;;    buflen=0
proc str.upcase string, buflen
	push	edi ecx

	;ECX = buflen, error if buflen=0
	mov	ecx, [buflen]
	cmp	ecx, 0
	je	.error_buflen_zero

	;EDI = string
	mov	edi, [string]

	;loop for each character
.convert_char:

	;conver character and move EDI to next one
	mov	al, [edi]
	cmp	al, 'a'
	jb	@f
	cmp	al, 'z'
	ja	@f
	and	byte [edi], not 20h
@@:	inc	edi

	;check if we reached end of buffer
	dec	ecx
	jz	.buffer_overflow

	;check if we reached ending zero
	cmp	al, 0
	jnz	.convert_char

	;string buffer wasn't terminated with zero
.buffer_overflow:

.done:	mov	eax, [string]
	clc

.r:	pop	ecx edi
	ret
.rc:	stc
	jmp	.r

.error_buflen_zero:
	mov	eax, ERR_ZERO_SIZE
	jmp	.rc
endp