;; @name stream.peek.byte
;; @desc
;;    Pre-reads byte from stream..
;;    Returned byte still remain in stream, and will be readen by subsequent read.
;;    Blocks, if there is no data available.
;; @arg stream
;;    Handle of stream.
;; @ret
;;    CF set on error, otherwise
;;    OF set if EOF was reached, otherwise
;;    EAX = byte, zero extended to dword.
;; @note Stream must have peek method supported, or read buffer.
;; @err ERR_INVALID_HANDLE
;;    Not a stream handle.
;; @err ERR_HANDLE_NO_BUFFER
;;    Stream doesn't have read buffer associated, nor peek method supported.
;; @err ERR_HANDLE_NOT_PEEKABLE
;;    Stream is not peekable.
;; @err ERR_HANDLE_CORRUPT
;;    Stream is corrupt, you are overwriting stream object somewhere
;;    (or it's error in FASMLIB).
proc stream.peek.byte stream
	libcall stream.peek, [stream], 1, 0
	jc	.r
	jo	.r
	movzx	eax, byte [eax]
	cmp	eax, eax		;clear OF
	clc
.r:	ret
endp