;; @name stream.seek
;; @desc
;;    Moves data pointer to given position.
;;    Distance to move is 64bit value.
;; @arg stream
;;    Handle of stream.
;; @arg distlow
;;    Low order dword of distance to move.
;; @arg disthigh
;;    High order dword of distance to move.
;; @arg mode
;;    Mode of seeking.
;;    0 = seek from beginning.
;;    1 = seek relative distance from current position.
;;    2 = seek from end (distance should be negative value).
;; @ret
;;    CF set on error
;; @err ERR_NEGATIVE_SEEK
;;    Seek to negative offset.
;; @err ERR_INVALID_MODE
;;    mode > 2
;; @err ERR_HANDLE_NOT_SEEKABLE
;;    This type of stream doesn't support seeking.
;; @note This procedure doesn't return new position in file.
;;       Use {stream.tell} to find out new position.
;; @dep {stream seek method}
proc stream.seek stream, distlow, disthigh, mode

	;error if mode is >2
	cmp	[mode], 2
	ja	.error_invalid_mode

	;EAX = stream virtual table
	mov	eax, [stream]
	mov	eax, [eax + STREAM.vtab]

	;check if this is proper stream
	cmp	[eax + STREAM_VTAB.sign], 'STRM'
	jne	.error_not_stream

	;EAX = seek method, error if not present
	mov	eax, [eax + STREAM_VTAB.seek]
	cmp	eax, 0
	je	.error_stream_not_seekable

	;call method
	libcall eax, [stream], [distlow], [disthigh], [mode]
	jc	.rc

	;to make sure no one will use return value
	mov	eax, -1

.rnc:	clc
.r:	ret
.rc:	stc
	jmp	.r

.error_not_stream:
	mov	eax, ERR_INVALID_HANDLE
	jmp	.rc

.error_invalid_mode:
	mov	eax, ERR_INVALID_MODE
	jmp	.rc

.error_stream_not_seekable:
	mov	eax, ERR_HANDLE_NOT_SEEKABLE
	jmp	.rc
endp