;; @name text.extread.dec_b
;; @desc
;;    Reads unsigned decimal byte from stream.
;;    This is extended version, for simpler usage see {text.read.dec_b}.
;;    Reads decimal digit characters ('0'-'9') until it reaches non-digit, EOF,
;;    or until resulting value overflows.
;; @arg stream
;;    Handle of stream from which to read.
;; @arg maxlen
;;    Maximum number of digits to read.
;;    0 if not specified.
;; @arg skipbl
;;    If 1, procedure first skips initial blank characters (space, tab, EOL).
;;    These characters don't count into maxlen size.
;; @arg len
;;    Pointer to dword variable, that will be set to number of digits read.
;;    Ignored if 0.
;;    When skipbl=1, skipped initial blank characters don't count.
;; @ret
;;    CF set on error, otherwise
;;    EAX = readen byte zero-extended to dword
;; @err ERR_HANDLE_EOF
;;    EOF reached. len is set to 0.
;; @err ERR_OUT_OF_RANGE
;;    value > 255.
;;    In this case all digit characters remain on stream.
;;    len contains number of characters readen until overflow.
;; @err ERR_NOT_DECIMAL
;;    First character is not decimal digit.
;;    Character remains on stream.
;;    len is set to 0.
;; @err ERR_ZERO_VALUE
;;    maxlen=0.
;;    len is set to 0.
;; @err ERR_INVALID_MODE
;;    skipbl > 1.
;;    len is set to 0.
;; @note Stream must have a read buffer.
;; @note If stream has static read buffer, it must be big enough to hold all digits
;;       of number.
;; @nodep text.extread.dec2d_size
proc text.extread.dec_b stream, maxlen, skipbl, len
	libcall text.extread.dec2d_size, [stream], 255, [maxlen], [skipbl], [len]
	ret
endp
