;; @name text.extread.line
;; @desc
;;     Reads one line of text from stream to buffer.
;;     This is extended version of procedure, for simpler version see {text.read.line}.
;; @arg stream
;;     Stream to read characters from.
;; @arg buffer
;;     Pointer to destination buffer, where the characters will be stored.
;; @arg buflen
;;     Length of destination buffer.
;; @arg maxlen
;;     Maximum number of characters to read.
;;     Ignored if 0.
;; @arg len
;;     Pointer to dword variable, which on return holds number of characters readen.
;;     This is same value as one returned in EAX, and is only useful on ERR_BUFFER_FULL.
;;     Ignored if 0.
;; @ret
;;    CF=1 on error, otherwise
;;    EAX = number of characters readen, and
;;    ZF=1 if EAX=0 (no characters readen).
;; @note Supports both LF and CRLF end of line combinations.
;; @note End of line character is not stored in resulting string.
;; @err ERR_ZERO_SIZE
;;    buflen=0.
;;    len is set to 0.
;; @err ERR_BUFFER_FULL
;;    Destiantion buffer is full.
;;    Only buflen characters readen, and string in destination buffer is
;;    NOT zero-terminated.
;;    len is set to buflen.
;; @err ERR_HANDLE_EOF
;;    EOF reached without reading anything. len is set to 0.
proc text.extread.line, stream, buffer, buflen, maxlen, len
	push ebx

	;read the line into buffer
	libcall text.extread.until, [stream], [buffer], [buflen], text.eol_chars, [maxlen], [len]
	jc	.rc

	mov	ebx, eax

	;being smart-ass, we can skip character 13 if it's there
	libcall text.fetch.char, [stream], 13
	jc	.rc

	;then let's fetch character 10
	libcall text.fetch.char, [stream], 10
	jc	.rc

	mov	eax, ebx

	clc
.r:	pop	ebx
	ret
.rc:	stc
	jmp	.r

endp