;; @name text.extwrite.d_hex
;; @desc
;;    Write unsigned dword to stream as hexadecimal string.
;; @arg stream
;;    Handle of stream to write to.
;; @arg value
;;    Value to write as hexadecimal string.
;; @arg padding
;;    Type of padding:
;;    type 0 is no padding,
;;    type 1 is prepending with spaces,
;;    type 2 is prepending with leading zeroes.
;; @arg padlen
;;    Length to which should output be padded.
;;    If output is already longer than this value before padding, then output is not padded.
;;    Ignored if padding = 0.
;; @ret
;;    CF=1 on error.
;; @err ERR_INVALID_MODE
;;    padding > 2
;; @err ERR_OUT_OF_RANGE
;;    padlen > 4096
;; @note This is extended version of procedure, for simpler usage see {text.write.d_hex}.
;; @note For signed version of this procedure, see {text.extwrite.id_hex}.
proc text.extwrite.d_hex stream, value, padding, padlen
local convlen dd ?

	push	ebx ecx

	;ECX = size of needed buffer, at least 16
	mov	ecx, [padlen]
	cmp	[padding], 0
	jne	@f
	xor	ecx, ecx
	@@:
	inc	ecx
	add	ecx, 3
	and	ecx, not 3  ;align to dword
	mov	eax, 16
	cmp	ecx, eax
	cmovb	ecx, eax

	;sanity-check for padlen > 4096, to prevent big allocation on stack
	cmp	ecx, 4096
	ja	.error_padlen_too_big

	;EBX = allocte padlen+1 buffer on stack
	sub	esp, ecx
	mov	ebx, esp

	;convert
	libcall extconv.d_hex, ebx, ecx, [value], [padding], [padlen], addr convlen
	jc	.error_convert

	;display
	libcall stream.write, [stream], ebx, [convlen]
	jc	.error_write

	;remove buffer from stack
	add	esp, ecx

.r:	pop	ebx
	ret
.rc:	stc
	jmp	.r

.error_write:
	add	esp, ecx
	jmp	.rc

.error_padlen_too_big:
	mov	eax, ERR_OUT_OF_RANGE
	jmp	.rc

.error_convert:
	add	esp, ecx
	;if returned error is ERR_BUFFER_FULL, it's FASMLIB bug
	cmp	eax, ERR_BUFFER_FULL
	jne	.rc
	mov	eax, ERR_FASMLIB
	jmp	.rc

endp