;; @name text.extwrite.w_hex
;; @desc
;;    Write unsigned word to stream as hexadecimal string.
;; @arg stream
;;    Handle of stream to write to.
;; @arg value
;;    Value to write as hexadecimal string.
;;    Only low order word of this dword is used, rest is ignored.
;; @arg padding
;;    Type of padding:
;;    type 0 is no padding,
;;    type 1 is prepending with spaces,
;;    type 2 is prepending with leading zeroes.
;; @arg padlen
;;    Length to which should output be padded.
;;    If output is already longer than this value before padding, then output is not padded.
;;    Ignored if padding = 0.
;; @ret
;;    CF=1 on error.
;; @err ERR_INVALID_MODE
;;    padding > 2
;; @err ERR_OUT_OF_RANGE
;;    padlen > 4096
;; @note This is extended version of procedure, for simpler usage see {text.write.w_hex}.
;; @note For signed version of this procedure, see {text.extwrite.iw_hex}.
proc text.extwrite.w_hex stream, value, padding, padlen
	movzx	eax, word [value]
	libcall text.extwrite.d_hex, [stream], eax, [padding], [padlen]
	ret
endp
