;; @name text.fetch.char
;; @desc
;;    Reads specified ASCII character off the stream.
;;    If there is another character on stream, then nothing is read and ZF=1 is returned.
;; @arg stream
;;    Handle of stream to read from.
;; @arg char
;;    Character to fetch from stream.
;;    Low order byte of this dword is used, rest is ignored.
;; @ret
;;    CF set on error, otherwise
;;    ZF set if the character was fetched
;; @note For standard input, you can call {stdin.fetch.char} instead of passing {stdin} as argument.
proc text.fetch.char stream, char

	;peek character
	libcall stream.peek.byte, [stream]
	jc	.r
	jo	.r

	;return ZF=0, if it isn't the one we want
	cmp	al, byte [char]
	jne	.rnc

	;skip the character
	libcall stream.skip, [stream], 1
	jc	.r
	jo	.error_internal

	;return ZF=1
	cmp	al, al

.rnc:	clc
.r:	ret
.rc:	stc
	jmp	.r

.error_internal:
	mov	eax, ERR_FASMLIB
	jmp	.rc
endp