;; @name text.read.chars
;; @desc
;;     Reads from stream to buffer all characters which are in given
;;     character list.
;;     This is simpler version of procedure, for extended version see {text.extread.chars}.
;; @arg stream
;;     Stream to read characters from.
;; @arg buffer
;;     Pointer to destination buffer, where the characters will be stored.
;; @arg buflen
;;     Length of destination buffer.
;; @arg charlist
;;     Null-terminated string containing list of characters to read.
;; @ret
;;    CF=1 on error, otherwise
;;    EAX = number of characters readen, and
;;    ZF=1 if EAX=0 (no characters readen).
;; @note
;;    This function never reads character 0 from stream.
;; @err ERR_ZERO_SIZE
;;    buflen=0.
;; @err ERR_BUFFER_FULL
;;    Destiantion buffer is full.
;;    Only buflen characters readen, and string in destination buffer is
;;    NOT zero-terminated.
;; @err ERR_HANDLE_EOF
;;    EOF reached without reading anything.
proc text.read.chars stream, buffer, buflen, charlist
	libcall text.extread.chars, [stream], [buffer], [buflen], [charlist], 0, 0
	ret
endp