;; @name text.read.dec_d
;; @desc
;;    Reads unsigned decimal dword from stream.
;;    This is simple version, for more options see {text.extread.dec_d}.
;;    Reads decimal digit characters ('0'-'9') until it reaches non-digit, EOF,
;;    or until resulting value overflows.
;; @arg stream
;;    Handle of stream from which to read.
;; @ret
;;    CF set on error, otherwise
;;    EAX = readen dword
;; @err ERR_HANDLE_EOF
;;    EOF reached.
;; @err ERR_OUT_OF_RANGE
;;    value > 4294967295.
;;    In this case all digit characters remain on stream.
;; @err ERR_NOT_DECIMAL
;;    First character is not decimal digit.
;;    Character remains on stream.
;; @note Skips inital blank characters (space, tab, EOL).
;; @note Stream must have a read buffer.
;; @note If stream has static read buffer, it must be big enough to hold all digits
;;       of number.
;; @nodep text.extread.dec2d_size
proc text.read.dec_d stream
	libcall text.extread.dec2d_size, [stream], 4294967295, 0, 1, 0
	ret
endp