;; @name text.read.hex_id
;; @desc
;;    Reads signed hexadecimal dword from stream.
;;    This is simple version, for more options see {text.extread.hex_id}.
;;    Reads optional sign ('-' or '+') and hexadecimal digit characters ('0'-'9','a'-'f','A'-'F') until
;;    it reaches non-digit, EOF, or until resulting value overflows.
;; @arg stream
;;    Handle of stream from which to read.
;; @ret
;;    CF set on error, otherwise
;;    EAX = readen dword
;; @err ERR_HANDLE_EOF
;;    EOF reached.
;; @err ERR_OUT_OF_RANGE
;;    value < -80000000 or value > 7FFFFFFF.
;;    In this case sign character and all digit characters remain on stream.
;; @err ERR_NOT_HEXADECIMAL
;;    First character (after sign) is not hexadecimal digit.
;;    Character remains on stream.
;; @note Skips inital blank characters (space, tab, EOL).
;; @note Stream must have a read buffer.
;; @note If stream has static read buffer, it must be big enough to hold all digits
;;       of number.
;; @nodep text.extread.hex2id_size
proc text.read.hex_id stream
	libcall text.extread.hex2id_size, [stream], 80000000h, 0, 1, 1, 0
	ret
endp